@@ -1,7 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Core
--- Copyright   :  (c) 2011 diagrams-core team (see LICENSE)
+-- Copyright   :  (c) 2011-2015 diagrams-core team (see LICENSE)
 -- License     :  BSD-style (see LICENSE)
 -- Maintainer  :  diagrams-discuss@googlegroups.com
 --
@@ -22,7 +22,7 @@
 -- The diagrams library relies heavily on custom types and classes. Many
 -- of the relevant definitions are in the "Diagrams.Core.Types" module.
 -- Indeed the definition of the diagram type @QDiagram@ is contained in:
--- 'Diagrams.Core.Types.QDiagram'. 
+-- 'Diagrams.Core.Types.QDiagram'.
 --
 -- The best place to start when learning
 -- about diagrams\' types is the user manual:
@@ -32,72 +32,72 @@
 --
 -- * "Diagrams.Core.Types"
 --
---     * @Annotation@, 
---     * @UpAnnots b v m@, @DownAnnots v@,
---     * @QDiaLeaf b v m@, @Measure v@,
---     * @Subdiagram b v m@,  @SubMap b v m@,
---     * @Prim b v@, @Backend b v@, 
---     * @DNode b v a@, @DTree b v a@,
---     * @RNode b v a@, @RTree b v a@,
---     * @NullBackend@, @Renderable t b@,
---     * @D v@.
+--     * @'Annotation'@,
+--     * @'UpAnnots' b v n m@, @'DownAnnots' v n@,
+--     * @'QDiaLeaf' b v n m@, @'Measure' n@,
+--     * @'Subdiagram' b v n m@,  @'SubMap' b v n m@,
+--     * @'Prim' b v n@, @'Backend' b v n@,
+--     * @'DNode' b v n a@, @'DTree' b v n a@,
+--     * @'RNode' b v n a@, @'RTree' b v n a@,
+--     * @'NullBackend'@, @'Renderable' t b@,
+--     * @'D' v n@.
 --
 -- * "Diagrams.Core.Envelope"
 --
---     * @Envelope v@, @Enveloped v@,
---     * @OrderedField s@.
+--     * @'Envelope' v n@, @'Enveloped' a@,
+--     * @'OrderedField' s@.
 --
 -- * "Diagrams.Core.Juxtapose"
 --
---     * @Juxtaposable a@.
+--     * @'Juxtaposable' a@.
 --
 -- * "Diagrams.Core.Names"
 --
---     * @AName@, @Name@, @IsName a@,
---     * @Qualifiable q@.
+--     * @'AName'@, @'Name'@, @'IsName' a@,
+--     * @'Qualifiable' q@.
 --
 -- * "Diagrams.Core.HasOrigin"
 --
---     * @HasOrigin t@.
---
--- * "Diagrams.Core.Points"
---
---     * @Point v@.
+--     * @'HasOrigin' t@.
 --
 -- * "Diagrams.Core.Query"
 --
---     * @Query v m@.
+--     * @'Query' v n m@.
 --
 -- *  "Diagrams.Core.Style"
 --
---     * @AttributeClass a@, @Attribute v@,
---     * @Style v@, @HasStyle@.
+--     * @'AttributeClass' a@, @'Attribute' v n@,
+--     * @'Style' v n@, @'HasStyle'@.
 --
 -- * "Diagrams.Core.Trace"
 --
---     * @SortedList a@,
---     * @Trace v@, @Traced a@.
+--     * @'SortedList' a@,
+--     * @'Trace' v n@, @'Traced' a@.
 --
 -- * "Diagrams.Core.Transform"
 --
---     * @u :-: v@, @HasLinearMap@,
---     * @Transformation v@, @Transformable t@,
---     * @TransInv t@.
+--     * @u ':-:' v@, @'HasLinearMap'@, @'HasBasis'@
+--     * @'Transformation' v n@, @'Transformable' t@,
+--     * @'TransInv' t@.
 --
 -- * "Diagrams.Core.V"
 --
---     * @V a@.
+--     * @'V' a@,
+--     * @'N' a@,
+--     * @'Vn' a@,
+--     * @'InSpace' v n a@,
+--     * @'SameSpace' a b@.
 -----------------------------------------------------------------------------
 
 module Diagrams.Core
        ( -- * Associated vector spaces
 
-         V
+         V, N, Vn, InSpace, SameSpace
 
          -- * Points
 
        , Point, origin, (*.)
-       , _relative
+       , relative
 
          -- * Transformations
 
@@ -105,6 +105,7 @@ module Diagrams.Core
        , basis
        , dimension
        , determinant
+       , isReflection
 
          -- ** Invertible linear transformations
        , (:-:), (<->), linv, lapp
@@ -129,6 +130,7 @@ module Diagrams.Core
          -- ** Translational invariance
 
        , TransInv(TransInv)
+       , eye
 
          -- * Names
 
@@ -147,11 +149,12 @@ module Diagrams.Core
          -- * Attributes and styles
 
        , AttributeClass
-       , Attribute, mkAttr, mkTAttr, mkGTAttr, unwrapAttr
+       , Attribute (..)
 
        , Style, HasStyle(..)
-       , getAttr, combineAttr
-       , applyAttr, applyTAttr, applyGTAttr
+       , getAttr
+       , atAttr, atMAttr, atTAttr
+       , applyAttr, applyMAttr, applyTAttr
 
          -- * Envelopes
 
@@ -159,7 +162,7 @@ module Diagrams.Core
        , appEnvelope, onEnvelope, mkEnvelope
        , Enveloped(..)
        , envelopeVMay, envelopeV, envelopePMay, envelopeP
-       , diameter, radius
+       , diameter, radius, size
 
          -- * Traces
 
@@ -184,7 +187,7 @@ module Diagrams.Core
 
        , Query(..)
 
-         -- * Primtives
+         -- * Primitives
 
        , Prim(..)
 
@@ -201,6 +204,8 @@ module Diagrams.Core
        , localize
 
        , href
+       , opacityGroup
+       , groupOpacity
 
        , setEnvelope, setTrace
 
@@ -213,12 +218,18 @@ module Diagrams.Core
        , location
        , subPoint
 
-         -- * Measurements
-       , Measure(..)
-       , fromOutput
-       , toOutput
-       , atMost
+         -- ** Measurements
+
+       , Measured
+       , Measure
+       , fromMeasured
+       , output
+       , local
+       , global
+       , normalized
+       , scaleLocal
        , atLeast
+       , atMost
 
          -- * Backends
 
@@ -235,7 +246,9 @@ module Diagrams.Core
          -- * Convenience classes
 
        , HasLinearMap
+       , HasBasis
        , OrderedField
+       , TypeableFloat
        , Monoid'
 
        ) where
@@ -244,6 +257,7 @@ import           Diagrams.Core.Compile
 import           Diagrams.Core.Envelope
 import           Diagrams.Core.HasOrigin
 import           Diagrams.Core.Juxtapose
+import           Diagrams.Core.Measure
 import           Diagrams.Core.Names
 import           Diagrams.Core.Points
 import           Diagrams.Core.Query
