@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
@@ -6,7 +7,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Core.Compile
--- Copyright   :  (c) 2013 diagrams-core team (see LICENSE)
+-- Copyright   :  (c) 2013-2015 diagrams-core team (see LICENSE)
 -- License     :  BSD-style (see LICENSE)
 -- Maintainer  :  diagrams-discuss@googlegroups.com
 --
@@ -30,12 +31,10 @@ module Diagrams.Core.Compile
 
   , toDTree
   , fromDTree
-  , styleToOutput
-  , toOutput
   )
   where
 
-import           Data.Data
+import           Data.Typeable
 import qualified Data.List.NonEmpty        as NEL
 import           Data.Maybe                (fromMaybe)
 import           Data.Monoid.Coproduct
@@ -44,21 +43,28 @@ import           Data.Monoid.WithSemigroup (Monoid')
 import           Data.Semigroup
 import           Data.Tree
 import           Data.Tree.DUAL
-import           Data.VectorSpace
+
 import           Diagrams.Core.Envelope    (OrderedField, diameter)
-import           Diagrams.Core.Style
 import           Diagrams.Core.Transform
 import           Diagrams.Core.Types
+import           Diagrams.Core.Style
 
-emptyDTree :: Tree (DNode b v a)
+import           Linear.Metric hiding (qd)
+
+-- Typeable1 is a depreciated synonym in ghc > 707
+#if __GLASGOW_HASKELL__ >= 707
+#define Typeable1 Typeable
+#endif
+
+emptyDTree :: Tree (DNode b v n a)
 emptyDTree = Node DEmpty []
 
 uncurry3 :: (a -> b -> c -> r) -> (a, b, c) -> r
 uncurry3 f (x, y, z) = f x y z
 
 -- | Convert a @QDiagram@ into a raw tree.
-toDTree :: HasLinearMap v => Scalar v -> Scalar v -> QDiagram b v m
-                          -> Maybe (DTree b v Annotation)
+toDTree :: (HasLinearMap v, Floating n, Typeable n)
+        => n -> n -> QDiagram b v n m -> Maybe (DTree b v n Annotation)
 toDTree g n (QD qd)
   = foldDUAL
 
@@ -75,8 +81,7 @@ toDTree g n (QD qd)
                -- the continuation, convert the result to a DTree, and
                -- splice it in, adding a DDelay node to mark the point
                -- of the splice.
-               (Node DDelay . (:[]) . fromMaybe emptyDTree . toDTree g n
-                            . ($ (d, g, n)) . uncurry3)
+               (Node DDelay . (:[]) . fromMaybe emptyDTree . toDTree g n . ($ (d, g, n)) . uncurry3)
       )
 
       -- u-only leaves --> empty DTree. We don't care about the
@@ -108,10 +113,11 @@ toDTree g n (QD qd)
 -- | Convert a @DTree@ to an @RTree@ which can be used dirctly by backends.
 --   A @DTree@ includes nodes of type @DTransform (Transformation v)@;
 --   in the @RTree@ transform is pushed down until it reaches a primitive node.
-fromDTree :: HasLinearMap v => DTree b v Annotation -> RTree b v Annotation
+fromDTree :: forall b v n. (Floating n, HasLinearMap v)
+          => DTree b v n Annotation -> RTree b v n Annotation
 fromDTree = fromDTree' mempty
   where
-    fromDTree' :: HasLinearMap v => Transformation v -> DTree b v Annotation -> RTree b v Annotation
+    fromDTree' :: HasLinearMap v => Transformation v n -> DTree b v n Annotation -> RTree b v n Annotation
     -- We put the accumulated transformation (accTr) and the prim
     -- into an RPrim node.
     fromDTree' accTr (Node (DPrim p) _)
@@ -145,10 +151,11 @@ fromDTree = fromDTree' mempty
 --   transformation used to convert the diagram from local to output
 --   units.
 toRTree
-  :: (HasLinearMap v, InnerSpace v, Data v, Data (Scalar v), OrderedField (Scalar v), Monoid m, Semigroup m)
-  => Transformation v -> QDiagram b v m -> RTree b v Annotation
+  :: (HasLinearMap v, Metric v, Typeable1 v, Typeable n,
+      OrderedField n, Monoid m, Semigroup m)
+  => Transformation v n -> QDiagram b v n m -> RTree b v n Annotation
 toRTree globalToOutput d
-  = (fmap . onRStyle) (styleToOutput gToO nToO)
+  = (fmap . onRStyle) (unmeasureAttrs gToO nToO)
   . fromDTree
   . fromMaybe (Node DEmpty [])
   . toDTree gToO nToO
@@ -160,50 +167,14 @@ toRTree globalToOutput d
     -- of product of diameters along each basis direction.  Note at
     -- this point the diagram has already had the globalToOutput
     -- transformation applied, so output = global = local units.
-    nToO = product (map (\v -> diameter v d) basis) ** (1 / fromIntegral (dimension d))
+    nToO = product (map (`diameter` d) basis) ** (1 / fromIntegral (dimension d))
 
 -- | Apply a style transformation on 'RStyle' nodes; the identity for
 --   other 'RNode's.
-onRStyle :: (Style v -> Style v) -> (RNode b v a -> RNode b v a)
+onRStyle :: (Style v n -> Style v n) -> RNode b v n a -> RNode b v n a
 onRStyle f (RStyle s) = RStyle (f s)
 onRStyle _ n          = n
 
--- | Convert all 'Measure' values to 'Output' units.  The arguments
---   are, respectively, the scaling factor from global units to output
---   units, and from normalized units to output units.  It is assumed
---   that local units are identical to output units (which will be the
---   case if all transformations have been fully pushed down and
---   applied). Normalized units are based on a logical diagram size of
---   1 x 1.
-styleToOutput
-  :: forall v. (Data v, Data (Scalar v), Num (Scalar v), Ord (Scalar v), Fractional (Scalar v))
-  => Scalar v -> Scalar v -> Style v -> Style v
-styleToOutput globalToOutput normToOutput =
-  gmapAttrs (toOutput globalToOutput normToOutput :: Measure v -> Measure v)
-
--- | Convert an aribrary 'Measure' to 'Output' units.
-toOutput :: forall v. (Data v, Data (Scalar v), Num (Scalar v), Ord (Scalar v), Fractional (Scalar v))
-  => Scalar v -> Scalar v -> Measure v -> Measure v
-toOutput g n m =
-  case m of
-     m'@(Output _) -> m'
-     Local s       -> Output s
-     Global s      -> Output (g * s)
-     Normalized s  -> Output (n * s)
-
-     MinM m1 m2    -> outBin min (toOutput g n m1) (toOutput g n m2)
-     MaxM m1 m2    -> outBin max (toOutput g n m1) (toOutput g n m2)
-     ZeroM         -> Output 0
-     NegateM m'    -> outUn negate (toOutput g n m')
-     PlusM m1 m2   -> outBin (+) (toOutput g n m1) (toOutput g n m2)
-     ScaleM s m'   -> outUn (s*) (toOutput g n m')
-  where
-    outUn  op (Output o1)             = Output (op o1)
-    outUn  _  _ = error "outUn: The sky is falling!"
-    outBin op (Output o1) (Output o2) = Output (o1 `op` o2)
-    outBin _ _ _ = error "outBin: Both skies are falling!"
-
-
 --------------------------------------------------
 
 -- | Render a diagram, returning also the transformation which was
@@ -212,21 +183,16 @@ toOutput g n m =
 --   transformation can be used, for example, to convert output/screen
 --   coordinates back into diagram coordinates.  See also 'adjustDia'.
 renderDiaT
-  :: ( Backend b v
-     , HasLinearMap v, InnerSpace v, Data v
-     , OrderedField (Scalar v), Data (Scalar v)
-     , Monoid' m
-     )
-  => b -> Options b v -> QDiagram b v m -> (Transformation v, Result b v)
+  :: (Backend b v n , HasLinearMap v, Metric v, Typeable1 v,
+      Typeable n, OrderedField n, Monoid' m)
+  => b -> Options b v n -> QDiagram b v n m -> (Transformation v n, Result b v n)
 renderDiaT b opts d = (g2o, renderRTree b opts' . toRTree g2o $ d')
   where (opts', g2o, d') = adjustDia b opts d
 
 -- | Render a diagram.
 renderDia
-  :: ( Backend b v
-     , InnerSpace v, Data v
-     , OrderedField (Scalar v), Data (Scalar v)
-     , Monoid' m
-     )
-          => b -> Options b v -> QDiagram b v m -> Result b v
+  :: (Backend b v n , HasLinearMap v, Metric v, Typeable1 v,
+      Typeable n, OrderedField n, Monoid' m)
+  => b -> Options b v n -> QDiagram b v n m -> Result b v n
 renderDia b opts d = snd (renderDiaT b opts d)
+
