@@ -1,7 +1,8 @@
-{-# LANGUAGE FlexibleContexts     #-}
-{-# LANGUAGE FlexibleInstances    #-}
-{-# LANGUAGE TypeFamilies         #-}
-{-# LANGUAGE UndecidableInstances #-}
+{-# LANGUAGE FlexibleContexts      #-}
+{-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE UndecidableInstances  #-}
 
 -- The UndecidableInstances flag is needed under 6.12.3 for the
 -- HasOrigin (a,b) instance.
@@ -25,12 +26,13 @@ module Diagrams.Core.HasOrigin
 import qualified Data.Map             as M
 import qualified Data.Set             as S
 
-import           Data.AffineSpace     ((.-.), (.-^))
-import           Data.VectorSpace
-
-import           Diagrams.Core.Points
+import           Diagrams.Core.Measure
+import           Diagrams.Core.Points ()
 import           Diagrams.Core.V
 
+import           Linear.Affine
+import           Linear.Vector
+
 -- | Class of types which have an intrinsic notion of a \"local
 --   origin\", i.e. things which are not invariant under translation,
 --   and which allow the origin to be moved.
@@ -40,13 +42,13 @@ import           Diagrams.Core.V
 --   are instances of both we should have the identity
 --
 --   @
---   moveOriginTo (origin .^+ v) === translate (negateV v)
+--   moveOriginTo (origin .^+ v) === translate (negated v)
 --   @
 --
 --   The reason is that some things (e.g. vectors, 'Trail's) are
 --   transformable but are translationally invariant, i.e. have no
 --   origin.
-class VectorSpace (V t) => HasOrigin t where
+class HasOrigin t where
 
   -- | Move the local origin to another point.
   --
@@ -54,10 +56,10 @@ class VectorSpace (V t) => HasOrigin t where
   --   (for types which are also 'Transformable'); moving the origin
   --   itself while leaving the object \"fixed\" is dual to fixing the
   --   origin and translating the diagram.
-  moveOriginTo :: Point (V t) -> t -> t
+  moveOriginTo :: Point (V t) (N t) -> t -> t
 
 -- | Move the local origin by a relative vector.
-moveOriginBy :: HasOrigin t => V t -> t -> t
+moveOriginBy :: (V t ~ v, N t ~ n, Num n, HasOrigin t) => v n -> t -> t
 moveOriginBy = moveOriginTo . P
 
 -- | Translate the object by the translation that sends the origin to
@@ -74,26 +76,30 @@ moveOriginBy = moveOriginTo . P
 --   @
 --   moveTo (origin .^+ v) === translate v
 --   @
-moveTo :: HasOrigin t => Point (V t) -> t -> t
+moveTo :: (InSpace v n t, HasOrigin t) => Point v n -> t -> t
 moveTo = moveOriginBy . (origin .-.)
 
 -- | A flipped variant of 'moveTo', provided for convenience.  Useful
 --   when writing a function which takes a point as an argument, such
 --   as when using 'withName' and friends.
-place :: HasOrigin t => t -> Point (V t) -> t
+place :: (InSpace v n t, HasOrigin t) => t -> Point v n -> t
 place = flip moveTo
 
-instance VectorSpace v => HasOrigin (Point v) where
+instance HasOrigin t => HasOrigin (Measured n t) where
+  moveOriginTo = fmap . moveOriginTo
+
+instance (Additive v, Num n) => HasOrigin (Point v n) where
   moveOriginTo (P u) p = p .-^ u
 
-instance (HasOrigin a, HasOrigin b, V a ~ V b) => HasOrigin (a,b) where
+instance (HasOrigin t, HasOrigin s, SameSpace s t) => HasOrigin (s, t) where
   moveOriginTo p (x,y) = (moveOriginTo p x, moveOriginTo p y)
 
-instance HasOrigin a => HasOrigin [a] where
+instance HasOrigin t => HasOrigin [t] where
   moveOriginTo = map . moveOriginTo
 
-instance (HasOrigin a, Ord a) => HasOrigin (S.Set a) where
+instance (HasOrigin t, Ord t) => HasOrigin (S.Set t) where
   moveOriginTo = S.map . moveOriginTo
 
-instance HasOrigin a => HasOrigin (M.Map k a) where
+instance HasOrigin t => HasOrigin (M.Map k t) where
   moveOriginTo = M.map . moveOriginTo
+
