@@ -1,7 +1,8 @@
-{-# LANGUAGE FlexibleContexts
-           , UndecidableInstances
-           , TypeFamilies
-  #-}
+{-# LANGUAGE FlexibleContexts      #-}
+{-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE UndecidableInstances  #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Core.Juxtapose
@@ -18,16 +19,18 @@ module Diagrams.Core.Juxtapose
        ( Juxtaposable(..), juxtaposeDefault
        ) where
 
-import           Data.Functor ((<$>))
-import qualified Data.Map as M
-import qualified Data.Set as S
-
-import           Data.VectorSpace
+import           Control.Applicative
+import qualified Data.Map                as M
+import qualified Data.Set                as S
 
 import           Diagrams.Core.Envelope
+import           Diagrams.Core.Measure
 import           Diagrams.Core.HasOrigin
 import           Diagrams.Core.V
 
+import           Linear.Metric
+import           Linear.Vector
+
 -- | Class of things which can be placed \"next to\" other things, for some
 --   appropriate notion of \"next to\".
 class Juxtaposable a where
@@ -38,23 +41,23 @@ class Juxtaposable a where
   --   @a2@; @a1@'s local origin becomes @a2@'s new local origin.  The
   --   result is just a translated version of @a2@.  (In particular,
   --   this operation does not /combine/ @a1@ and @a2@ in any way.)
-  juxtapose :: V a -> a -> a -> a
+  juxtapose :: Vn a -> a -> a -> a
 
 -- | Default implementation of 'juxtapose' for things which are
 --   instances of 'Enveloped' and 'HasOrigin'.  If either envelope is
 --   empty, the second object is returned unchanged.
-juxtaposeDefault :: (Enveloped a, HasOrigin a) => V a -> a -> a -> a
+juxtaposeDefault :: (Enveloped a, HasOrigin a) => Vn a -> a -> a -> a
 juxtaposeDefault v a1 a2 =
   case (mv1, mv2) of
     (Just v1, Just v2) -> moveOriginBy (v1 ^+^ v2) a2
     _                  -> a2
-  where mv1 = negateV <$> envelopeVMay v a1
-        mv2 = envelopeVMay (negateV v) a2
+  where mv1 = negated <$> envelopeVMay v a1
+        mv2 = envelopeVMay (negated v) a2
 
-instance (InnerSpace v, OrderedField (Scalar v)) => Juxtaposable (Envelope v) where
+instance (Metric v, OrderedField n) => Juxtaposable (Envelope v n) where
   juxtapose = juxtaposeDefault
 
-instance (Enveloped a, HasOrigin a, Enveloped b, HasOrigin b, V a ~ V b)
+instance (Enveloped a, HasOrigin a, Enveloped b, HasOrigin b, V a ~ V b, N a ~ N b)
          => Juxtaposable (a,b) where
   juxtapose = juxtaposeDefault
 
@@ -69,3 +72,7 @@ instance (Enveloped b, HasOrigin b, Ord b) => Juxtaposable (S.Set b) where
 
 instance Juxtaposable a => Juxtaposable (b -> a) where
   juxtapose v f1 f2 b = juxtapose v (f1 b) (f2 b)
+
+instance Juxtaposable a => Juxtaposable (Measured n a) where
+  juxtapose v = liftA2 (juxtapose v)
+
